@@ -10,15 +10,16 @@ https://github.com/zabuldon/teslajsonpy
 """
 from functools import partial
 import logging
-from typing import Any, Dict, Text
-
 import random
-from aiohttp import ClientResponse, web
+from typing import Any, Dict, Optional
+
+from aiohttp import web
+import httpx
 from authcaptureproxy import AuthCaptureProxy, return_timer_countdown_refresh_html
 from authcaptureproxy.examples.modifiers import find_regex_urls
-from authcaptureproxy.helper import prepend_url
-from yarl import URL
+from authcaptureproxy.helper import get_content_type, prepend_url
 import multidict
+from yarl import URL
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -35,11 +36,11 @@ class TeslaProxy(AuthCaptureProxy):
 
         """
         super().__init__(URL(proxy_url), URL(host_url))
-        self._config_flow_id = None
-        self._callback_url = None
-        self.waf_retry = 0
-        self.waf_limit = 30
-        self.tests = {"test_url": self.test_url}
+        self._config_flow_id: Optional[str] = None
+        self._callback_url: Optional[str] = None
+        self.waf_retry: int = 0
+        self.waf_limit: int = 30
+        self.tests: Dict[str, str] = {"test_url": self.test_url}
 
         self.headers = {
             "x-tesla-user-agent": "TeslaApp/3.10.9-433/adff2e065/android/10",
@@ -58,42 +59,44 @@ class TeslaProxy(AuthCaptureProxy):
                 },
             }
         )
+        self.redirect_filters = {"url": ["^.*/static/404.html$"]}
 
     async def test_url(
         self,
-        resp: ClientResponse,
-        data: Dict[Text, Any],
-        query: Dict[Text, Any],  # pylint: disable=unused-argument
+        resp: httpx.Response,
+        data: Dict[str, Any],
+        query: Dict[str, Any],  # pylint: disable=unused-argument
     ):
         """Test for a successful Tesla URL.
 
         https://tesla-api.timdorr.com/api-basics/authentication#step-2-obtain-an-authorization-code
 
         Args:
-            resp (ClientResponse): The aiohttp response.
-            data (Dict[Text, Any]): Dictionary of all post data captured through proxy with overwrites for duplicate keys.
-            query (Dict[Text, Any]): Dictionary of all query data with overwrites for duplicate keys.
+            resp (httpx.Response): The httpx response.
+            data (Dict[str, Any]): Dictionary of all post data captured through proxy with overwrites for duplicate keys.
+            query (Dict[str, Any]): Dictionary of all query data with overwrites for duplicate keys.
 
         Returns
-            Optional[Union[URL, Text]]: URL for a http 302 redirect or Text to display on success. None indicates test did not pass.
+            Optional[Union[URL, str]]: URL for a http 302 redirect or str to display on success. None indicates test did not pass.
 
         """
-        code: Text = ""
+        code: str = ""
         if resp.url.path == "/void/callback":
             code = resp.url.query.get("code")
         if resp.url.path == "/static/404.html":
-            code = URL(resp.history[-1].url).query.get("code")
+            code = URL(str(resp.history[-1].url)).query.get("code")
         if code:
             username = data.get("identity")
             self._callback_url = self.init_query.get("callback_url")
             self.waf_retry = 0
             _LOGGER.debug("Success! Oauth code %s for %s captured.", code, username)
+            await self.session.aclose()
             # 302 redirect
             return URL(self._callback_url).update_query(
                 {"code": code, "username": username, "domain": self._host_url.host}
             )
-        if resp.content_type == "text/html":
-            text = await resp.text()
+        if get_content_type(resp) == "text/html":
+            text = resp.text
             if "<noscript>Please enable JavaScript to view the page content." in text:
                 _LOGGER.debug("WAF discovered %s times in a row.", self.waf_retry)
                 self.waf_retry += 1
@@ -105,21 +108,21 @@ class TeslaProxy(AuthCaptureProxy):
                     False,
                 )
             self.waf_retry = 0
-        if resp.content_type == "application/json":
-            text = await resp.json()
+        if get_content_type(resp) == "application/json":
+            text = resp.json()
             _LOGGER.debug("Json response: %s", text)
 
-    async def prepend_relative_urls(self, base_url: URL, html: Text) -> Text:
+    async def prepend_relative_urls(self, base_url: URL, html: str) -> str:
         """Prepend relative urls with url host.
 
         This is intended to be used for to place the proxy_url in front of relative urls in src="/
 
         Args:
             base_url (URL): Base URL to prepend
-            html (Text): Text to replace
+            html (str): text to replace
 
         Returns
-            Text: Replaced text
+            str: Replaced text
 
         """
         if not base_url:
@@ -142,17 +145,17 @@ class TeslaProxy(AuthCaptureProxy):
         self.waf_retry = 0
         await super().reset_data()
 
-    async def prepend_i18n_path(self, base_url: URL, html: Text) -> Text:
+    async def prepend_i18n_path(self, base_url: URL, html: str) -> str:
         """Prepend path for i18n loadPath so it'll reach the proxy.
 
         This is intended to be used for to place the proxy_url path in front of relative urls for loadPath in i18next.
 
         Args:
             base_url (URL): Base URL to prepend
-            html (Text): Text to replace
+            html (str): text to replace
 
         Returns
-            Text: Replaced text
+            str: Replaced text
 
         """
         if not base_url:
@@ -161,9 +164,7 @@ class TeslaProxy(AuthCaptureProxy):
 
         return await find_regex_urls(
             partial(prepend_url, base_url, encoded=True),
-            {
-                "method_func": r"""(?:loadPath:)\s*?["']([^"']*)[\"\']""",
-            },
+            {"method_func": r"""(?:loadPath:)\s*?["']([^"']*)[\"\']"""},
             html=html,
         )
 
@@ -172,16 +173,22 @@ class TeslaProxy(AuthCaptureProxy):
     ) -> multidict.MultiDict:
         """Modify headers.
 
+        Return modified headers based on site and request. To disable auto header generation,
+        pass in a key const.SKIP_AUTO_HEADERS with a list of keys to not generate.
+
+        For example, to prevent User-Agent generation: {SKIP_AUTO_HEADERS : ["User-Agent"]}
+
         Args:
             site (URL): URL of the next host request.
             request (web.Request): Proxy directed request. This will need to be changed for the actual host request.
 
         Returns
-            multidict.MultiDict: Headers after modifications
+            dict: Headers after modifications
 
         """
         result = await super().modify_headers(site, request)
         method = request.method
+        # result.update({SKIP_AUTO_HEADERS: ["User-Agent"]})
         if (
             str(site.path) == "/oauth2/v3/authorize/mfa/verify"
             and method == "POST"
@@ -189,5 +196,5 @@ class TeslaProxy(AuthCaptureProxy):
         ):
             # allow post json to autogenerate headers.
             # https://github.com/timdorr/tesla-api/discussions/316.
-            return multidict.MultiDict({})
-        return multidict.MultiDict(result)
+            return {}
+        return result
